/*
 * Copyright (c) 2006, The Joust Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 * - Neither the name of the Joust Project nor the names of its
 *   contributors may be used to endorse or promote products derived
 *   from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * File created by keithkml
 */

package net.kano.joustsim;

import java.util.Collection;
import java.util.List;
import java.util.ArrayList;

public final class TestTools {
  private TestTools() { }

  public static <E> E findOnlyInstance(Collection<?> hit,
      Class<E> cls) {
    E inst = null;
    for (Object controller : hit) {
      if (cls.isInstance(controller)) {
        if (inst != null) {
          throw new IllegalArgumentException("Two instances of " + cls + ": "
              + hit);
        }
        inst = cls.cast(controller);
      }
    }
    return inst;
  }

  public static boolean hasInstances(List<?> hit, Class<?>... classes) {
    boolean worked = true;
    int good = 0;
    Classes:
    for (Class<?> cls : classes) {
      for (int i = good; i < hit.size(); i++) {
        Object controller = hit.get(i);

        if (cls.isInstance(controller)) {
          good = i;
          continue Classes;
        }
      }
      worked = false;
      break;
    }
    return worked;
  }

  public static <E,F extends E> List<F> findInstances(List<E> hit, Class<F> cls) {
    List<F> list = new ArrayList<F>();
    for (E controller : hit) {
      if (cls.isInstance(controller)) list.add(cls.cast(controller));
    }
    return list;
  }
}
